# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for pkgdb messages """

import unittest

from fedmsg.tests.test_meta import Base

from .common import add_doc


class TestPkgdbACLUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes these messages when an ACL changes on a package.
    """

    expected_title = "pkgdb.acl.update"
    expected_subti = ("ralph changed ralph's 'watchbugzilla' permission on "
                      "python-sh (EL-6) to 'Awaiting Review'")
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/python-sh/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['python-sh'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['python-sh/acls/EL-6/watchbugzilla/ralph'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357576703.125622,
        "topic": "org.fedoraproject.stg.pkgdb.acl.update",
        "msg": {
            "status": "Awaiting Review",
            "username": "ralph",
            "package_listing": {
                "point_of_contact": "grover",
                "package": {
                    "upstreamurl": None,
                    "name": "python-sh",
                    "description": None,
                    "reviewurl": None,
                    "summary": "Python module to simplify calling "
                    "shell commands"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "agent": "ralph",
            "acl": "watchbugzilla"
        }
    }


class TestPkgdbPackageNew(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when a new package is added to the DB.  This
    typically happens near the end of the Package Review Process as a
    result of a `SCM Admin Request
    <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_.
    """
    expected_title = "pkgdb.package.new"
    expected_subti = "ralph added a new package 'php-zmq' (devel)"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/create'])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1357580533.5999,
        "topic": "org.fedoraproject.stg.pkgdb.package.new",
        "msg": {
            "package_listing": {
                "point_of_contact": "lmacken",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora",
                    "publishurltemplate": None,
                    "version": "19",
                    "disttag": ".f19",
                    "branchname": "devel"
                },
                "specfile": None
            },
            "agent": "ralph"
        }
    }


class TestPkgdbOwnerUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when a package gets an new owner.  (It is
    also published when a package is orphaned; the 'owner' field will have
    the string 'orphan' as its value.)
    """
    expected_title = "pkgdb.owner.update"
    expected_subti = "ralph changed owner of php-zmq (EL-6) to 'orphan'"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/owner/EL-6'])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1357580533.5999,
        "topic": "org.fedoraproject.stg.pkgdb.owner.update",
        "msg": {
            "package_listing": {
                "point_of_contact": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "agent": "ralph"
        }
    }


class TestLegacyPkgdbACLRequestToggle(Base):
    """ The old Fedora Package DB1 published this message when an ACL request
    was toggled on a package.
    """
    expected_title = "pkgdb.acl.request.toggle"
    expected_subti = "ralph has requested 'commit' on php-zmq (EL-6)"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/acls/EL-6/commit/ralph'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.acl.request.toggle",
        "msg": {
            "acl_action": "requested",
            "package_listing": {
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "acl_status": "Awaiting Review",
            "agent": "ralph",
            "acl": "commit"
        }
    }


class TestLegacyPkgdbPackageUpdate(Base):
    """ Test old school messages. """
    expected_title = "pkgdb.package.update"
    expected_subti = "ralph made some updates to php-zmq"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/update'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.package.update",
        "msg": {
            "acl_action": "requested",
            "package": "php-zmq",
            "agent": "ralph",
        },
    }


class TestPkgdbPackageUpdateStatus(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when the status of a package is updated.
    """
    expected_title = "pkgdb.package.update.status"
    expected_subti = "ralph unretired guake in F-18"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/guake/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['guake'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['guake/update'])
    msg = {
        "username": "apache",
        "i": 144,
        "timestamp": 1379605523.496933,
        "msg_id": "2013-c131fb95-0a2e-4426-95c3-09766e017d29",
        "topic": "org.fedoraproject.dev.pkgdb.package.update.status",
        "msg": {
            "status": "Approved",
            "package_listing": {
                "package": {
                    "status": "Approved",
                    "upstream_url": "http://guake.org",
                    "name": "guake",
                    "creation_date": 1379619917.0,
                    "summary": "Top down terminal for GNOME",
                    "review_url": "https://bugzilla.redhat.com/450189"
                },
                "collection": {
                    "pendingurltemplate": None,
                    "publishurltemplate": None,
                    "branchname": "F-18",
                    "name": "Fedora",
                    "version": "18"
                },
                "point_of_contact": "pingou"
            },
            "prev_status": "Retired",
            "agent": "ralph",
            "package_name": "guake"
        }
    }


class TestPkgdbPackageUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when metadata for a package is updated.
    """
    expected_title = "pkgdb.package.update"
    expected_subti = "pkgdb_updater updated: summary, description of guake"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/guake/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "1ff483b03adb34142ac55a5efecfa71b0149d57566f86d969905005b0ab98def"
        "?s=64&d=retro")
    expected_packages = set(['guake'])
    expected_usernames = set(['pkgdb_updater'])
    expected_objects = set(['guake/update'])
    msg = {
        "username": "apache",
        "i": 144,
        "timestamp": 1379605523.496933,
        "msg_id": "2013-c131fb95-0a2e-4426-95c3-09766e017d29",
        "topic": "org.fedoraproject.dev.pkgdb.package.update",
        "msg": {
            "package": {
                    "status": "Approved",
                    "upstream_url": "http://guake.org",
                    "name": "guake",
                    "creation_date": 1379619917.0,
                    "summary": "Top down terminal for GNOME",
                    "review_url": "https://bugzilla.redhat.com/450189"
                },
            "agent": "pkgdb_updater",
            "fields": ["summary", "description"],
        }
    }


class LegacyTestPkgdbBranchClone(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages on this topic when a new branch is cloned for a
    package.
    """
    expected_title = "pkgdb.branch.clone"
    expected_subti = "ralph branched php-zmq f18 from devel"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/branch'])

    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.branch.clone",
        "msg": {
            "package": "php-zmq",
            "branch": "f18",
            "master": "devel",
            "agent": "ralph",
        },
    }


class TestLegacyPkgdbCritpathUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages on this topic when the critical path status of a
    package changes (when it is either added, or removed from the critical
    path).  For example:
    """
    expected_title = "pkgdb.critpath.update"
    expected_subti = "ralph altered the critpath status for some packages"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set([])

    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357581512.006664,
        "topic": "org.fedoraproject.stg.pkgdb.critpath.update",
        "msg": {
            "package_listing_ids": [],
            "agent": "ralph",
            "critpath": True,
        },
    }


class TestPkgdbPackageUpdateStatus2(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes this message when the status of a package is updated.
    Here's an example of a package being retired:
    """
    expected_title = "pkgdb.package.update.status"
    expected_subti = "till retired libvmime07 in master"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/libvmime07/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "272bbf32f26ca494a78673f873bb62e8f3deb9f9b53213ceac3c2a144de4784a"
        "?s=64&d=retro")
    expected_packages = set(['libvmime07'])
    expected_usernames = set(['till'])
    expected_objects = set(['libvmime07/update'])
    msg = {
        "source_name": "datanommer",
        "i": 7,
        "timestamp": 1412710605.0,
        "msg_id": "2014-78aa26ee-d2e5-4446-b4a4-73948704d73e",
        "topic": "org.fedoraproject.prod.pkgdb.package.update.status",
        "source_version": "0.6.4",
        "msg": {
            "status": "Retired",
            "package_listing": {
                "status": "Retired",
                "point_of_contact": "orphan",
                "package": {
                    "status": "Approved",
                    "upstream_url": "http://www.zarafa.com/wiki/index.php/Libvmime_patches",
                    "description": "VMime is a powerful C++ class ...",
                    "creation_date": 1400070978.0,
                    "acls": [],
                    "summary": "A powerful C++ class ...",
                    "review_url": None,
                    "name": "libvmime07"
                },
                "collection": {
                    "status": "Under Development",
                    "dist_tag": ".fc22",
                    "koji_name": "rawhide",
                    "name": "Fedora",
                    "version": "devel",
                    "branchname": "master"
                },
                "acls": [
                    {
                        "fas_name": "robert",
                        "status": "Approved",
                        "acl": "watchcommits"
                    },
                    {
                        "fas_name": "robert",
                        "status": "Approved",
                        "acl": "watchbugzilla"
                    },
                    {
                        "fas_name": "robert",
                        "status": "Obsolete",
                        "acl": "commit"
                    },
                    {
                        "fas_name": "robert",
                        "status": "Obsolete",
                        "acl": "approveacls"
                    }
                ],
                "critpath": False,
                "status_change": 1412710603.0
            },
            "prev_status": "Orphaned",
            "package_name": "libvmime07",
            "agent": "till"
        }
    }


class TestLegacyPkgdbPackageRetire(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages on this topic when a package is retired.  For example:
    """
    expected_title = "pkgdb.package.retire"
    expected_subti = "ralph retired php-zmq (EL-6)!"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/retire'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357583297.886945,
        "topic": "org.fedoraproject.stg.pkgdb.package.retire",
        "msg": {
            "package_listing": {
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            },
            "retirement": "retired",
            "agent": "ralph"
        }
    }


class LegacyTestPkgdbUserRemove(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    when a user is removed from a package ACL.
    """
    expected_title = "pkgdb.acl.user.remove"
    expected_subti = "ralph removed ralph from php-zmq (EL-6, F18)"
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/php-zmq/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set(['php-zmq'])
    expected_usernames = set(['ralph'])
    expected_objects = set(['php-zmq/remove/ralph'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1357583297.886945,
        "topic": "org.fedoraproject.stg.pkgdb.acl.user.remove",
        "msg": {
            "package_listings": [{
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora EPEL",
                    "publishurltemplate": None,
                    "version": "6",
                    "disttag": ".el6",
                    "branchname": "EL-6"
                },
                "specfile": None
            }, {
                "owner": "orphan",
                "package": {
                    "upstreamurl": None,
                    "name": "php-zmq",
                    "description": None,
                    "reviewurl": None,
                    "summary": "PHP 0MQ/zmq/zeromq extension"
                },
                "qacontact": None,
                "collection": {
                    "pendingurltemplate": None,
                    "name": "Fedora",
                    "publishurltemplate": None,
                    "version": "18",
                    "disttag": ".f18",
                    "branchname": "F18"
                },
                "specfile": None
            }],
            "collections": [
                # This actually has stuff in it in prod.
            ],
            "username": "ralph",
            "agent": "ralph",
        }
    }


class TestPkgdbBranchStart(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when branching starts.
    """
    expected_title = "pkgdb.branch.start"
    expected_subti = "ralph started a branch of F-19 from devel"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 1,
        u'timestamp': 1379606342.105066,
        u'msg_id': u'2013-0eaf6d98-6259-4e1c-a113-e2c9284a6082',
        u'topic':
        u'org.fedoraproject.dev.pkgdb.branch.start',
        u'msg': {
            u'collection_from': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'devel',
                u'name': u'Fedora',
                u'version': u'devel'
            },
            u'collection_to': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19'
            },
            u'agent': u'ralph',
        },
    }


class TestLegacyPkgdbBranchStart(Base):
    """ This just tests a funny case where 'agent' is a list.. """
    expected_title = "pkgdb.branch.start"
    expected_subti = "ralph started a branch of F-19 from devel"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 1,
        u'timestamp': 1379606342.105066,
        u'msg_id': u'2013-0eaf6d98-6259-4e1c-a113-e2c9284a6082',
        u'topic':
        u'org.fedoraproject.dev.pkgdb.branch.start',
        u'msg': {
            u'collection_from': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'devel',
                u'name': u'Fedora',
                u'version': u'devel'
            },
            u'collection_to': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19'
            },
            u'agent': [u'ralph'],
        },
    }


class TestPkgdbBranchComplete(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when branching completes.
    """
    expected_title = "pkgdb.branch.complete"
    expected_subti = "ralph's branch of F-19 from devel completed"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 1,
        u'timestamp': 1379606342.105066,
        u'msg_id': u'2013-0eaf6d98-6259-4e1c-a113-e2c9284a6082',
        u'topic':
        u'org.fedoraproject.dev.pkgdb.branch.complete',
        u'msg': {
            u'collection_from': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'devel',
                u'name': u'Fedora',
                u'version': u'devel'
            },
            u'collection_to': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19'
            },
            u'agent': u'ralph',
        },
    }


class TestPkgdbCollectionNew(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin creates a new collection.
    """
    expected_title = "pkgdb.collection.new"
    expected_subti = "ralph created a new collection for Fedora 19"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 3,
        u'timestamp': 1379607327.474346,
        u'msg_id': u'2013-68fd388e-60ca-4cf6-888d-b51161798496',
        u'topic': u'org.fedoraproject.dev.pkgdb.collection.new',
        u'msg': {
            u'collection': {
                u'pendingurltemplate': None,
                u'publishurltemplate': None,
                u'branchname': u'F-19',
                u'name': u'Fedora',
                u'version': u'19',
            },
            u'agent': u'ralph',
        }
    }


class TestPkgdbCollectionUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin creates a new collection.
    """
    expected_title = "pkgdb.collection.update"
    expected_subti = ("ralph updated the following fields of the Fedora 18 "
                      "collection: name, version")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c?s=64&d=retro")
    expected_packages = set()
    expected_usernames = set(['ralph'])
    expected_objects = set()
    msg = {
        u'username': u'threebean',
        u'i': 27,
        u'timestamp': 1379607692.198447,
        u'msg_id': u'2013-478a321f-ddfc-4d4c-adeb-c777619da15a',
        u'topic': u'org.fedoraproject.dev.pkgdb.collection.update',
        u'msg': {
            u'fields': [
                u'name',
                u'version',
            ],
            u'collection': {
                u'pendingurltemplate': u'http://.....',
                u'publishurltemplate': u'http://.....',
                u'branchname': u'f18_b',
                u'name': u'Fedora',
                u'version': u'18'
            },
            u'agent': u'ralph',
        }
    }


class TestPkgdbDeletePackage(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin **deletes a package** all
    together.
    """
    expected_title = "pkgdb.package.delete"
    expected_subti = ("ausil deleted the 'pipelight' package from the pkgdb")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "a89b57d99dcf12d40ec2b9fb05910b90293b13b0b87415208bedc897bc18a354"
        "?s=64&d=retro")
    expected_packages = set(['pipelight'])
    expected_usernames = set(['ausil'])
    expected_objects = set(['pipelight/package/delete'])
    msg = {
        "i": 46,
        "msg_id": "2014-9372bf63-8e32-4257-82ec-38fb5226763a",
        "source_name": "datanommer",
        "source_version": "0.6.4",
        "timestamp": 1408377920.0,
        "topic": "org.fedoraproject.prod.pkgdb.package.delete",
        "msg": {
            "agent": "ausil",
            "package": {
                "acls": [
                    {
                        "acls": [
                            {
                                "acl": "commit",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "awjb",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "master",
                            "dist_tag": ".fc22",
                            "koji_name": "rawhide",
                            "name": "Fedora",
                            "status": "Under Development",
                            "version": "devel"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1404850009.0,
                            "description": "",
                            "name": "pipelight",
                            "review_url": "https://bugzilla.redhat.com/"
                            "1117403",
                            "status": "Approved",
                            "summary": "NPAPI Wrapper Plugin for using "
                            "Windows plugins in Linux browsers",
                            "upstream_url": "http://pipelight.net/"
                        },
                        "point_of_contact": "besser82",
                        "status": "Approved",
                        "status_change": 1404850010.0
                    },
                    {
                        "acls": [
                            {
                                "acl": "commit",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "awjb",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "f19",
                            "dist_tag": ".fc19",
                            "koji_name": "f19",
                            "name": "Fedora",
                            "status": "Active",
                            "version": "19"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1404850009.0,
                            "description": "",
                            "name": "pipelight",
                            "review_url": "https://bugzilla.redhat.com/"
                            "1117403",
                            "status": "Approved",
                            "summary": "NPAPI Wrapper Plugin for using "
                            "Windows plugins in Linux browsers",
                            "upstream_url": "http://pipelight.net/"
                        },
                        "point_of_contact": "besser82",
                        "status": "Approved",
                        "status_change": 1404850009.0
                    },
                    {
                        "acls": [
                            {
                                "acl": "commit",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "awjb",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "f20",
                            "dist_tag": ".fc20",
                            "koji_name": "f20",
                            "name": "Fedora",
                            "status": "Active",
                            "version": "20"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1404850009.0,
                            "description": "",
                            "name": "pipelight",
                            "review_url": "https://bugzilla.redhat.com/"
                            "1117403",
                            "status": "Approved",
                            "summary": "NPAPI Wrapper Plugin for using "
                            "Windows plugins in Linux browsers",
                            "upstream_url": "http://pipelight.net/"
                        },
                        "point_of_contact": "besser82",
                        "status": "Approved",
                        "status_change": 1404850010.0
                    },
                    {
                        "acls": [
                            {
                                "acl": "commit",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "awjb",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "epel7",
                            "dist_tag": ".el7",
                            "koji_name": "epel7",
                            "name": "Fedora EPEL",
                            "status": "Under Development",
                            "version": "7"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1404850009.0,
                            "description": "",
                            "name": "pipelight",
                            "review_url": "https://bugzilla.redhat.com/"
                            "1117403",
                            "status": "Approved",
                            "summary": "NPAPI Wrapper Plugin for using "
                            "Windows plugins in Linux browsers",
                            "upstream_url": "http://pipelight.net/"
                        },
                        "point_of_contact": "besser82",
                        "status": "Approved",
                        "status_change": 1404850009.0
                    },
                    {
                        "acls": [
                            {
                                "acl": "watchcommits",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "besser82",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "awjb",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "awjb",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "f21",
                            "dist_tag": ".fc21",
                            "koji_name": "f21",
                            "name": "Fedora",
                            "status": "Under Development",
                            "version": "21"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1404850009.0,
                            "description": "",
                            "name": "pipelight",
                            "review_url": "https://bugzilla.redhat.com/"
                            "1117403",
                            "status": "Approved",
                            "summary": "NPAPI Wrapper Plugin for using "
                            "Windows plugins in Linux browsers",
                            "upstream_url": "http://pipelight.net/"
                        },
                        "point_of_contact": "besser82",
                        "status": "Approved",
                        "status_change": 1404997736.0
                    }
                ],
                "creation_date": 1404850009.0,
                "description": "",
                "name": "pipelight",
                "review_url": "https://bugzilla.redhat.com/1117403",
                "status": "Approved",
                "summary": "NPAPI Wrapper Plugin for using "
                "Windows plugins in Linux browsers",
                "upstream_url": "http://pipelight.net/"
            }
        },
    }


class TestPkgdbDeleteBranch(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin **deletes a branch** of a
    particular package.
    """
    expected_title = "pkgdb.package.branch.delete"
    expected_subti = "ausil deleted the f21 branch of the 'pipelight' package"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "a89b57d99dcf12d40ec2b9fb05910b90293b13b0b87415208bedc897bc18a354"
        "?s=64&d=retro")
    expected_packages = set(['pipelight'])
    expected_usernames = set(['ausil'])
    expected_objects = set(['pipelight/f21/delete'])
    msg = {
        "i": 45,
        "msg_id": "2014-fba4c0ac-f5ba-446f-bf70-94200e2d286f",
        "source_name": "datanommer",
        "source_version": "0.6.4",
        "timestamp": 1408377920.0,
        "topic": "org.fedoraproject.prod.pkgdb.package.branch.delete",
        "msg": {
            "agent": "ausil",
            "package_listing": {
                "acls": [
                    {
                        "acl": "watchcommits",
                        "fas_name": "besser82",
                        "status": "Approved"
                    },
                    {
                        "acl": "watchbugzilla",
                        "fas_name": "besser82",
                        "status": "Approved"
                    },
                    {
                        "acl": "approveacls",
                        "fas_name": "besser82",
                        "status": "Approved"
                    },
                    {
                        "acl": "commit",
                        "fas_name": "besser82",
                        "status": "Approved"
                    },
                    {
                        "acl": "watchcommits",
                        "fas_name": "awjb",
                        "status": "Approved"
                    },
                    {
                        "acl": "watchbugzilla",
                        "fas_name": "awjb",
                        "status": "Approved"
                    },
                    {
                        "acl": "approveacls",
                        "fas_name": "awjb",
                        "status": "Approved"
                    },
                    {
                        "acl": "commit",
                        "fas_name": "awjb",
                        "status": "Approved"
                    }
                ],
                "collection": {
                    "branchname": "f21",
                    "dist_tag": ".fc21",
                    "koji_name": "f21",
                    "name": "Fedora",
                    "status": "Under Development",
                    "version": "21"
                },
                "critpath": False,
                "package": {
                    "acls": [],
                    "creation_date": 1404850009.0,
                    "description": "",
                    "name": "pipelight",
                    "review_url": "https://bugzilla.redhat.com/1117403",
                    "status": "Approved",
                    "summary": "NPAPI Wrapper Plugin for using Windows "
                    "plugins in Linux browsers",
                    "upstream_url": "http://pipelight.net/"
                },
                "point_of_contact": "besser82",
                "status": "Approved",
                "status_change": 1404997736.0
            }
        },
    }


class TestPkgdbDeleteAcl(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin **deletes a branch** of a
    particular package.
    """
    expected_title = "pkgdb.acl.delete"
    expected_subti = ("ausil deleted awjb's watchcommits "
                      "rights from pipelight (f20)")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "a89b57d99dcf12d40ec2b9fb05910b90293b13b0b87415208bedc897bc18a354"
        "?s=64&d=retro")
    expected_packages = set(['pipelight'])
    expected_usernames = set(['ausil', 'awjb'])
    expected_objects = set(['pipelight/acls/f20/watchcommits/awjb'])
    msg = {
        "i": 23,
        "msg_id": "2014-f46f0993-ea29-4fe1-af44-807b863a12de",
        "source_name": "datanommer",
        "source_version": "0.6.4",
        "timestamp": 1408377918.0,
        "topic": "org.fedoraproject.prod.pkgdb.acl.delete",
        "msg": {
            "acl": {
                "acl": "watchcommits",
                "fas_name": "awjb",
                "packagelist": {
                    "collection": {
                        "branchname": "f20",
                        "dist_tag": ".fc20",
                        "koji_name": "f20",
                        "name": "Fedora",
                        "status": "Active",
                        "version": "20"
                    },
                    "critpath": False,
                    "package": {
                        "acls": [],
                        "creation_date": 1404850009.0,
                        "description": "",
                        "name": "pipelight",
                        "review_url": "https://bugzilla.redhat.com/1117403",
                        "status": "Approved",
                        "summary": "NPAPI Wrapper Plugin for using Windows "
                        "plugins in Linux browsers",
                        "upstream_url": "http://pipelight.net/"
                    },
                    "point_of_contact": "besser82",
                    "status": "Approved",
                    "status_change": 1404850010.0
                },
                "status": "Approved"
            },
            "agent": "ausil"
        },
    }


class TestPkgdbBranchRequest(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an user **requests a new branch** for
    a particular package.
    """
    expected_title = "pkgdb.package.branch.request"
    expected_subti = ("pingou requested branch epel7 for package R-BiocGenerics")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['R-BiocGenerics'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['R-BiocGenerics/branch/request/epel7/pingou'])
    msg = {
        "i": 1,
        "timestamp": 1408440084,
        "msg_id": "2014-250329a1-1ccf-4fc4-ad0c-e24365f89c0f",
        "topic": "org.fedoraproject.dev.pkgdb.package.branch.request",
        "msg": {
            "collection_to": {
              "status": "Under Development",
                "dist_tag": ".el7",
                "koji_name": "epel7",
                "name": "Fedora EPEL",
                "version": "7",
                "branchname": "epel7"
            },
            "package": {
                "status": "Approved",
                "upstream_url": None,
                "description": None,
                "summary": "Generic functions for Bioconductor",
                "acls": [],
                "creation_date": 1400063778.0,
                "review_url": None,
                "name": "R-BiocGenerics"
            },
            "agent": "pingou",
        },
    }


class TestPkgdbPackageRequest(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an user **requests a new package**
    to be added into Package DB.
    """
    expected_title = "pkgdb.package.new.request"
    expected_subti = ("pingou requested package guake on branch master")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/guake/"
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['guake'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['new/package/request/guake/master/pingou'])
    msg = {
        "i": 3,
        "timestamp": 1408440927,
        "msg_id": "2014-40c33929-8fa1-4cfb-9559-231af6d809aa",
        "topic": "org.fedoraproject.dev.pkgdb.package.new.request",
        "msg": {
            "info": {
                "pkg_summary": "A drop-down terminal for GNOME",
                "pkg_collection": "master",
                "pkg_review_url": "https://bugzilla.redhat.com/123",
                "pkg_upstream_url": "http://guake.org",
                "pkg_poc": "pingou",
                "pkg_status": "Approved",
                "pkg_name": "guake",
                "pkg_description": "",
                "pkg_critpath": False
            },
            "agent": "pingou",
            "collection": {
                "status": "Under Development",
                "dist_tag": ".fc22",
                "koji_name": "rawhide",
                "name": "Fedora",
                "version": "devel",
                "branchname": "master"
            },
            "package": None
        },
    }


class TestPkgdbAdminActionUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin **update the status of an
    Admin Action**.
    """
    expected_title = "pkgdb.admin.action.status.update"
    expected_subti = ("pingou changed pingou's package request for guake "
                      "in master from Awaiting Review to Approved")
    expected_link = "https://admin.fedoraproject.org/pkgdb/package/guake/"
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['guake'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['action/18/status/guake/master/pingou'])
    msg = {
        "i": 6,
        "timestamp": 1408441710,
        "msg_id": "2014-3a9cba3d-a1d0-4187-9fa0-995d54bf826d",
        "topic": "org.fedoraproject.dev.pkgdb.admin.action.status.update",
        "msg": {
            "action": {
                "info": {
                    'pkg_summary': u'A drop-down terminal for GNOME',
                    'pkg_status': u'Approved',
                    'pkg_collection': u'master',
                    'pkg_name': u'guake',
                    'pkg_review_url': u'https://bugzilla.redhat.com/123',
                    'pkg_description': u'',
                    'pkg_upstream_url': u'http://guake.org',
                    'pkg_poc': u'pingou',
                    'pkg_critpath': False
                },
                "status": "Approved",
                "package": None,
                "date_updated": 1408441710.0,
                "collection": {
                    "status": "Under Development",
                    "dist_tag": ".fc22",
                    "koji_name": "rawhide",
                    "name": "Fedora",
                    "version": "devel",
                    "branchname": "master"
                },
                "user": "pingou",
                "action": "request.package",
                "date_created": 1408433727.0,
                "from_collection": None,
                "id": 18
            },
            "old_status": "Awaiting Review",
            "new_status": "Approved",
            "agent": "pingou"
        },
    }

class TestPkgdbAdminActionUpdate_Denied(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when a request for a new branch/package is
    **denied/blocked**.
    """
    expected_title = "pkgdb.admin.action.status.update"
    expected_subti = ("pingou changed pingou's branch request for R-Biobase "
                      "in epel7 from Awaiting Review to Denied "
                      "with message: "
                      "This package should not be branched for EPEL7")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['R-Biobase'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['action/2/status/R-Biobase/epel7/pingou'])
    msg = {
        "i": 1,
        "timestamp": 1421830060,
        "msg_id": "2015-1acdeda2-e571-4071-a893-cc2b7ba46b02",
        "topic": "org.fedoraproject.dev.pkgdb.admin.action.status.update",
        "msg": {
          "action": {
            "info": {},
            "status": "Denied",
            "package": {
              "status": "Approved",
              "upstream_url": "http://bioconductor.org/packages/release/bioc/html/Biobase.html",
              "monitor": False,
              "description": "Base functions for Bioconductor (bioconductor.org). Biobase provides\nfunctions that are needed by many other Bioconductor packages or which\nreplace R functions.",
              "summary": "Base functions for Bioconductor",
              "acls": [],
              "creation_date": 1400063778.0,
              "review_url": None,
              "name": "R-Biobase"
            },
            "date_updated": 1421830060.0,
            "collection": {
              "status": "Under Development",
              "dist_tag": ".el7",
              "koji_name": "epel7",
              "name": "Fedora EPEL",
              "version": "7",
              "branchname": "epel7"
            },
            "user": "pingou",
            "action": "request.branch",
            "date_created": 1421227282.0,
            "message": "This package should not be branched for EPEL7",
            "id": 2
          },
          "old_status": "Awaiting Review",
          "new_status": "Denied",
          "agent": "pingou"
        }
      }


class TestPkgdbCritpathUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when an admin **updates the critpath flag on
    a package**.
    """
    expected_title = "pkgdb.package.critpath.update"
    expected_subti = ("pingou set the critpath flag on the "
                      "openbox package (f21)")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['openbox'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['openbox/critpath'])
    msg = {
        "msg_id": "2014-dbb1c4d3-2ffa-4212-9daa-1479bf11e8a4",
        "source_name": "datanommer",
        "source_version": "0.6.4",
        "timestamp": 1408557412.0,
        "topic": "org.fedoraproject.prod.pkgdb.package.critpath.update",
        "i": 35,
        "msg": {
            "agent": "pingou",
            "branches": [
                "f21"
            ],
            "critpath": True,
            "package": {
                "acls": [
                    {
                        "acls": [
                            {
                                "acl": "watchcommits",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "FC-5",
                            "dist_tag": ".fc5",
                            "koji_name": None,
                            "name": "Fedora",
                            "status": "EOL",
                            "version": "5"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1400070978.0,
                            "name": "openbox",
                            "review_url": None,
                            "status": "Approved",
                            "summary": "A highly configurable and "
                            "standards-compliant X11 window manager",
                            "upstream_url": None
                        },
                        "point_of_contact": "mlichvar",
                        "status": "Approved",
                        "status_change": 1400071632.0
                    },
                    {
                        "acls": [
                            {
                                "acl": "watchcommits",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            }
                        ],
                        "collection": {
                            "branchname": "FC-4",
                            "dist_tag": ".fc4",
                            "koji_name": None,
                            "name": "Fedora",
                            "status": "EOL",
                            "version": "4"
                        },
                        "critpath": False,
                        "package": {
                            "acls": [],
                            "creation_date": 1400070978.0,
                            "name": "openbox",
                            "review_url": None,
                            "status": "Approved",
                            "summary": "A highly configurable and "
                            "standards-compliant X11 window manager",
                            "upstream_url": None
                        },
                        "point_of_contact": "mlichvar",
                        "status": "Approved",
                        "status_change": 1400071632.0
                    },
                    {
                        "acls": [
                            {
                                "acl": "watchcommits",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "approveacls",
                                "fas_name": "mlichvar",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "cwickert",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "cwickert",
                                "status": "Approved"
                            },
                            {
                                "acl": "commit",
                                "fas_name": "cwickert",
                                "status": "Approved"
                            },
                            {
                                "acl": "watchcommits",
                                "fas_name": "athmane",
                                "status": "Obsolete"
                            },
                            {
                                "acl": "watchbugzilla",
                                "fas_name": "athmane",
                                "status": "Obsolete"
                            }
                        ],
                        "collection": {
                            "branchname": "f21",
                            "dist_tag": ".fc21",
                            "koji_name": "f21",
                            "name": "Fedora",
                            "status": "Under Development",
                            "version": "21"
                        },
                        "critpath": True,
                        "package": {
                            "acls": [],
                            "creation_date": 1400070978.0,
                            "name": "openbox",
                            "review_url": None,
                            "status": "Approved",
                            "summary": "A highly configurable and "
                            "standards-compliant X11 window manager",
                            "upstream_url": None
                        },
                        "point_of_contact": "mlichvar",
                        "status": "Approved",
                        "status_change": 1408557402.0
                    }
                ],
                "creation_date": 1400070978.0,
                "description": "Openbox is a window manager designed ...",
                "name": "openbox",
                "review_url": None,
                "status": "Approved",
                "summary": "A highly configurable and "
                "standards-compliant X11 window manager",
                "upstream_url": None
            }
        },
    }


class TestPkgdbPackageBranchNew(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when a **new branch** is created for a
    package.
    """
    expected_title = "pkgdb.package.branch.new"
    expected_subti = ("pingou created the branch 'epel7' for the package "
                "'R-BSgenome'")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['R-BSgenome'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['R-BSgenome/epel7/new'])
    msg = {
        "i": 1,
        "timestamp": 1408957258,
        "msg_id": "2014-645038a7-1f95-4a81-aa68-489c0ae55803",
        "topic": "org.fedoraproject.dev.pkgdb.package.branch.new",
        "msg": {
            "package_listing": {
                "status": "Approved",
                "package": {
                    "status": "Approved",
                    "upstream_url": None,
                    "description": None,
                    "summary": "Infrastructure shared by all the "
                    "Biostrings-based genome",
                    "acls": [],
                    "creation_date": 1400063778.0,
                    "review_url": None,
                    "name": "R-BSgenome"
                },
                "point_of_contact": "pingou",
                "collection": {
                    "status": "Under Development",
                    "dist_tag": ".el7",
                    "koji_name": "epel7",
                    "name": "Fedora EPEL",
                    "version": "7",
                    "branchname": "epel7"
                },
                "critpath": False,
                "status_change": 1408950057.0
            },
            "agent": "pingou",
            "package": {
                "status": "Approved",
                "upstream_url": None,
                "description": None,
                "summary": "Infrastructure shared by all the "
                "Biostrings-based genome",
                "acls": [],
                "creation_date": 1400063778.0,
                "review_url": None,
                "name": "R-BSgenome"
            }
        }
    }


class TestPkgdbPackageMonitorUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when someone changes the
    `monitoring <https://fedoraproject.org/wiki/Upstream_release_monitoring>`_
    status of a package.
    """
    expected_title = "pkgdb.package.monitor.update"
    expected_subti = ("pingou set the monitor flag of guake to False")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['guake'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['guake/monitor/false'])
    msg = {
        "username": "pingou",
        "i": 3,
        "timestamp": 1412957736,
        "msg_id": "2014-905aaa3c-483d-4923-95f7-56a8da38da62",
        "topic": "org.fedoraproject.dev.pkgdb.package.monitor.update",
        "msg": {
            "status": False,
            "agent": "pingou",
            "package": {
                "status": "Approved",
                "upstream_url": "http://www.guake.org/",
                "description": "Guake is a drop-down terminal for Gnome Desktop Environment,\nso you just need to press a key to invoke him,\nand press again to hide.",
                "summary": "Drop-down terminal for GNOME",
                "acls": [],
                "creation_date": 1397204290.0,
                "review_url": None,
                "name": "guake"
            }
        }
    }


class TestPkgdbPackageUnretireRequest(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when someone asks that a package is
    **unretired**.
    """
    expected_title = "pkgdb.package.unretire.request"
    expected_subti = ("moceap asks that netbeans-platform8 be unretired on "
        "master")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "360e1873c56312ea5866123f5ffaf4e07d419570b03af7f475c0d20c7501db06"
        "?s=64&d=retro")
    expected_packages = set(['netbeans-platform8'])
    expected_usernames = set(['moceap'])
    expected_objects = set(['netbeans-platform8/unretire/master'])
    msg = {
        'i': 1,
        'timestamp': 1427823120,
        'msg_id': '2015-bb28a398-e638-4509-9fa0-57d41c2ae0a4',
        'topic': 'org.fedoraproject.prod.pkgdb.package.unretire.request',
        'msg': {
            'collection': {
                'status': 'UnderDevelopment',
                'dist_tag': '.fc23',
                'koji_name': 'rawhide',
                'name': 'Fedora',
                'version': 'devel',
                'branchname': 'master'
            },
            'agent': 'moceap',
            'package': {
                'status': 'Approved',
                'upstream_url': None,
                'monitor': False,
                'summary': 'NetBeansPlatform8',
                'name': 'netbeans-platform8',
                'acls': [
                    {
                        'status': 'Retired',
                        'point_of_contact': 'orphan',
                        'package': {
                            'status': 'Approved',
                            'upstream_url': None,
                            'monitor': False,
                            'summary': 'NetBeansPlatform8',
                            'name': 'netbeans-platform8',
                            'acls': [],
                            'creation_date': 1400070978.0,
                            'review_url': None,
                            'description': 'NetBeansPlatformisaframeworkfordevelopmentof\nRichClientSwingApplications.Itcontainspowerful\nmodulesystemandasetofmodulesprovidingvarious\nfunctionalitiesneededforsimplificationof\ndevelopmentofmodulardesktopapplications.'
                        },
                        'collection': {
                            'status': 'UnderDevelopment',
                            'dist_tag': '.fc23',
                            'koji_name': 'rawhide',
                            'name': 'Fedora',
                            'version': 'devel',
                            'branchname': 'master'
                        },
                        'critpath': False,
                        'status_change': 1400071169.0
                    },
                    {
                        'status': 'Approved',
                        'point_of_contact': 'victorv',
                        'package': {
                            'status': 'Approved',
                            'upstream_url': None,
                            'monitor': False,
                            'summary': 'NetBeansPlatform8',
                            'name': 'netbeans-platform8',
                            'acls': [],
                            'creation_date': 1400070978.0,
                            'review_url': None,
                            'description': 'NetBeansPlatformisaframeworkfordevelopmentof\nRichClientSwingApplications.Itcontainspowerful\nmodulesystemandasetofmodulesprovidingvarious\nfunctionalitiesneededforsimplificationof\ndevelopmentofmodulardesktopapplications.'
                        },
                        'collection': {
                            'status': 'EOL',
                            'dist_tag': '.fc10',
                            'koji_name': 'dist-f10',
                            'name': 'Fedora',
                            'version': '10',
                            'branchname': 'f10'
                        },
                        'acls': [
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'watchcommits'
                            },
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'watchbugzilla'
                            },
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'commit'
                            },
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'approveacls'
                            }
                        ],
                        'critpath': False,
                        'status_change': 1400071253.0
                    },
                    {
                        'status': 'Approved',
                        'point_of_contact': 'victorv',
                        'package': {
                            'status': 'Approved',
                            'upstream_url': None,
                            'monitor': False,
                            'summary': 'NetBeansPlatform8',
                            'name': 'netbeans-platform8',
                            'acls': [

                            ],
                            'creation_date': 1400070978.0,
                            'review_url': None,
                            'description': 'NetBeansPlatformisaframeworkfordevelopmentof\nRichClientSwingApplications.Itcontainspowerful\nmodulesystemandasetofmodulesprovidingvarious\nfunctionalitiesneededforsimplificationof\ndevelopmentofmodulardesktopapplications.'
                        },
                        'collection': {
                            'status': 'EOL',
                            'dist_tag': '.fc11',
                            'koji_name': 'dist-f11',
                            'name': 'Fedora',
                            'version': '11',
                            'branchname': 'f11'
                        },
                        'acls': [
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'watchcommits'
                            },
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'watchbugzilla'
                            },
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'commit'
                            },
                            {
                                'fas_name': 'victorv',
                                'status': 'Approved',
                                'acl': 'approveacls'
                            }
                        ],
                        'critpath': False,
                        'status_change': 1400071427.0
                    },
                    {
                        'status': 'Orphaned',
                        'point_of_contact': 'orphan',
                        'package': {
                            'status': 'Approved',
                            'upstream_url': None,
                            'monitor': False,
                            'summary': 'NetBeansPlatform8',
                            'name': 'netbeans-platform8',
                            'acls': [

                            ],
                            'creation_date': 1400070978.0,
                            'review_url': None,
                            'description': 'NetBeansPlatformisaframeworkfordevelopmentof\nRichClientSwingApplications.Itcontainspowerful\nmodulesystemandasetofmodulesprovidingvarious\nfunctionalitiesneededforsimplificationof\ndevelopmentofmodulardesktopapplications.'
                        },
                        'collection': {
                            'status': 'EOL',
                            'dist_tag': '.fc12',
                            'koji_name': 'dist-f12',
                            'name': 'Fedora',
                            'version': '12',
                            'branchname': 'f12'
                        },
                        'critpath': False,
                        'status_change': 1400071659.0
                    }
                ],
                'creation_date': 1400070978.0,
                'review_url': None,
                'description': 'NetBeansPlatformisaframeworkfordevelopmentof\nRichClientSwingApplications.Itcontainspowerful\nmodulesystemandasetofmodulesprovidingvarious\nfunctionalitiesneededforsimplificationof\ndevelopmentofmodulardesktopapplications.'
            }
        }
    }


class TestPkgdbPackageKoscheiUpdate(Base):
    """ The Fedora `Package DB <https://admin.fedoraproject.org/pkgdb>`_
    publishes messages like these when someone changes the
    `koschei <https://apps.fedoraproject.org/koschei>`_ status of a package.
    """
    expected_title = "pkgdb.package.koschei.update"
    expected_subti = ("pingou set the koschei monitoring flag of guake to True")
    expected_icon = ("https://apps.fedoraproject.org/packages/images/icons/"
                     "package_128x128.png")
    expected_secondary_icon = (
        "https://seccdn.libravatar.org/avatar/"
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c"
        "?s=64&d=retro")
    expected_packages = set(['guake'])
    expected_usernames = set(['pingou'])
    expected_objects = set(['guake/koschei/true'])
    msg = {
        'username': u'pierrey',
        'i': 3,
        'timestamp': 1435313134,
        'msg_id': u'2015-7d0ecbd6-6892-4b34-98ff-b212d1fef74e',
        'topic': u'org.fedoraproject.dev.pkgdb.package.koschei.update',
        'msg': {
            'status': True,
            'agent': u'pingou',
            'package': {
                'status': u'Approved',
                'upstream_url': u'http: //www.guake.org/',
                'koschei_monitor': True,
                'monitor': False,
                'summary': u'Drop-downterminalforGNOME',
                'name': u'guake',
                    'acls': [
                ],
                'creation_date': 1400063778.0,
                'review_url': None,
                'description': 'Guake is a drop-down terminal for Gnome'
            }
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Pierre-Yves Chibon <pingou@pingoured.fr>
#
""" Tests for anitya messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestNewProject(Base):
    """ These messages are published when a new project is created on
    `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.project.new"
    expected_subti = 'pingou created a new project "foo"'
    expected_link = "https://pagure.io/foo"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo'])
    msg = {
      "i": 1,
      "timestamp": 1427445138,
      "msg_id": "2015-a37b0f13-aead-40eb-ab53-7af8e89e6854",
      "topic": "io.pagure.dev.pagure.project.new",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "agent": "pingou"
      }
    }


class TestNewIssue(Base):
    """ These messages are published when a ticket is opened against a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.new"
    expected_subti = 'pingou opened a new ticket foo#4: "bug"'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 2,
      "timestamp": 1427445817,
      "msg_id": "2015-a9e8a8d6-6197-48b8-9fc9-a03967a9d4bb",
      "topic": "io.pagure.dev.pagure.issue.new",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Open",
          "blocks": "",
          "title": "bug",
          "tags": [],
          "comments": [],
          "content": "report",
          "depends": "",
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestNewIssueComment(Base):
    """ These messages are published when a someone comments on a ticket
    opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.comment.added"
    expected_subti = 'pingou commented on ticket foo#4: "bug"'
    expected_link = "https://pagure.io/foo/issue/4#comment-380"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 1,
      "timestamp": 1427448698,
      "msg_id": "2015-539fc955-db5a-4bb5-a6a6-4a096a2d795d",
      "topic": "io.pagure.dev.pagure.issue.comment.added",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Open",
          "blocks": "",
          "title": "bug",
          "tags": [],
          "comments": [
            {
              "comment": "We should really fix this",
              "date_created": "1427445097",
              "id": 380,
              "parent": None,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              }
            }
          ],
          "content": "report",
          "depends": "",
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestNewIssueTag(Base):
    """ These messages are published when a someone adds a tag on a ticket
    opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.tag.added"
    expected_subti = 'pingou tagged ticket foo#4: bug and easyfix'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 4,
      "timestamp": 1427449624,
      "msg_id": "2015-64ac444e-915c-4a6c-820b-59e8daf14584",
      "topic": "io.pagure.dev.pagure.issue.tag.added",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "tags": [
          "easyfix",
          "bug",
        ],
        "issue": {
          "status": "Open",
          "blocks": "",
          "title": "bug",
          "tags": [],
          "comments": [],
          "content": "report",
          "depends": "",
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestRemovedIssueTag(Base):
    """ These messages are published when a someone removes a tag on a
    ticket opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.tag.removed"
    expected_subti = 'pingou removed the feature and future tags from '\
        'ticket foo#4'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 4,
      "timestamp": 1427450043,
      "msg_id": "2015-e1921852-c269-4c08-a611-dffe5c39417f",
      "topic": "io.pagure.dev.pagure.issue.tag.removed",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Open",
          "blocks": "",
          "title": "bug",
          "tags": "",
          "comments": [],
          "content": "report",
          "depends": "",
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou",
        "tags": [
          "feature",
          "future"
        ]
      }
    }


class TestAssignedIssue(Base):
    """ These messages are published when a someone is assigned to a
    ticket opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.assigned.added"
    expected_subti = 'pingou assigned ticket foo#4 to ralph'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 3,
      "timestamp": 1427450780,
      "msg_id": "2015-4ab5479a-1a99-4e26-a52f-e9e1ce423e40",
      "topic": "io.pagure.dev.pagure.issue.assigned.added",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Open",
          "blocks": "",
          "title": "bug",
          "tags": [],
          "comments": [],
          "content": "report",
          "assignee": {
            "fullname": "Ralph",
            "emails": [
              "ralph@fedoraproject.org"
            ],
            "name": "ralph"
          },
          "depends": "",
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestResetAssignedIssue(Base):
    """ These messages are published when a someone is reset the assignee
    of a ticket opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.assigned.reset"
    expected_subti = 'pingou reset the assignee of ticket foo#4'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 3,
      "timestamp": 1427453148,
      "msg_id": "2015-bc20fa0e-8baa-4b6a-ac44-c30b9e579da3",
      "topic": "io.pagure.dev.pagure.issue.assigned.reset",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Open",
          "blocks": "",
          "title": "bug",
          "tags": [],
          "comments": [],
          "content": "report",
          "assignee": None,
          "depends": "",
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestNewIssueDependency(Base):
    """ These messages are published when a someone is reset the assignee
    of a ticket opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.dependency.added"
    expected_subti = 'pingou added ticket foo#2 as a dependency of '\
    'ticket foo#4'
    expected_link = "https://pagure.io/foo/issue/2"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/2'])
    msg = {
      "i": 2,
      "timestamp": 1427453868,
      "msg_id": "2015-c8189e0c-ef22-4e72-92cc-9ffa68c35b7b",
      "topic": "io.pagure.dev.pagure.issue.dependency.added",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Open",
          "blocks": [4],
          "title": "bug",
          "tags": [],
          "comments": [],
          "content": "report",
          "assignee": None,
          "depends": "",
          "private": False,
          "date_created": "1427442076",
          "id": 2,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "added_dependency": 4,
        "agent": "pingou"
      }
    }


class TestRemovedIssueDependency(Base):
    """ These messages are published when a someone is reset the assignee
    of a ticket opened against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.dependency.removed"
    expected_subti = 'pingou removed ticket foo#4 as a dependency of '\
    'ticket foo#2'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 3,
      "timestamp": 1427454576,
      "msg_id": "2015-cb2e1acd-c6c7-4da4-ba99-c136954bb039",
      "topic": "io.pagure.dev.pagure.issue.dependency.removed",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "removed_dependency": 2,
        "issue": {
          "status": "Open",
          "blocks": [],
          "title": "bug",
          "tags": [
            "0.1"
          ],
          "comments": [],
          "content": "report",
          "assignee": None,
          "depends": [],
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestIssueEdit(Base):
    """ These messages are published when a someone edited a ticket opened
    against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.edit"
    expected_subti = 'pingou edited the private and status fields of '\
        'ticket foo#4'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 5,
      "timestamp": 1427454847,
      "msg_id": "2015-5755ee3a-43ba-4552-9423-8fe3b0a96662",
      "topic": "io.pagure.dev.pagure.issue.edit",
      "msg": {
        "fields": [
          "status",
          "private"
        ],
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Fixed",
          "blocks": [],
          "title": "bug",
          "tags": [
            "0.1"
          ],
          "comments": [],
          "content": "report",
          "assignee": None,
          "depends": [],
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestIssueEditStatus(Base):
    """ These messages are published when a someone edited a ticket opened
    against a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.edit"
    expected_subti = 'pingou set the status of ticket foo#4 to: Fixed'
    expected_link = "https://pagure.io/foo/issue/4"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo', 'issue/4'])
    msg = {
      "i": 5,
      "timestamp": 1427454847,
      "msg_id": "2015-5755ee3a-43ba-4552-9423-8fe3b0a96662",
      "topic": "io.pagure.dev.pagure.issue.edit",
      "msg": {
        "fields": [
          "status"
        ],
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "issue": {
          "status": "Fixed",
          "blocks": [],
          "title": "bug",
          "tags": [
            "0.1"
          ],
          "comments": [],
          "content": "report",
          "assignee": None,
          "depends": [],
          "private": False,
          "date_created": "1427442217",
          "id": 4,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestProjectEdit(Base):
    """ These messages are published when a someone edited a project on
    `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.project.edit"
    expected_subti = 'pingou edited the project_docs fields of '\
        'project foo'
    expected_link = "https://pagure.io/foo"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo'])
    msg = {
      "i": 2,
      "timestamp": 1427455343,
      "msg_id": "2015-3b53c72a-8585-4ddc-ba60-d7e969a0acbb",
      "topic": "io.pagure.dev.pagure.project.edit",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": False,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "fields": [
          "project_docs"
        ],
        "agent": "pingou"
      }
    }


class TestProjectUserAdded(Base):
    """ These messages are published when a someone gave admins rights on a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.project.user.added"
    expected_subti = 'pingou added "ralph" to project foo'
    expected_link = "https://pagure.io/foo"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo'])
    msg = {
      "i": 4,
      "timestamp": 1427455518,
      "msg_id": "2015-b3c2e568-259a-4b1f-9ecc-79493b89687a",
      "topic": "io.pagure.dev.pagure.project.user.added",
      "msg": {
        "new_user": "ralph",
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": False,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "agent": "pingou"
      }
    }


class TestProjectTagRemoved(Base):
    """ These messages are published when a someone removed a tag of a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.project.tag.removed"
    expected_subti = 'pingou removed tags "easyfix1" from project foo'
    expected_link = "https://pagure.io/foo"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo'])
    msg = {
      "i": 5,
      "timestamp": 1427455744,
      "msg_id": "2015-c6db4dd3-0a87-4eee-aab7-7758f566f36e",
      "topic": "io.pagure.dev.pagure.project.tag.removed",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": False,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "agent": "pingou",
        "tags": [
          "easyfix1"
        ]
      }
    }


class TestProjectTagEdited(Base):
    """ These messages are published when a someone edited a tag of a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.project.tag.edited"
    expected_subti = 'pingou altered tags on project foo from ' + \
        '"easyfix1" to "easyfix"'
    expected_link = "https://pagure.io/foo"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/foo'])
    msg = {
      "i": 2,
      "timestamp": 1427456487,
      "msg_id": "2015-79d76ac7-5c66-460d-8a39-17849e462a85",
      "topic": "io.pagure.dev.pagure.project.tag.edited",
      "msg": {
        "project": {
          "description": "bar",
          "parent": None,
          "project_docs": False,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427441537",
          "id": 7,
          "name": "foo"
        },
        "new_tag": "easyfix",
        "old_tag": "easyfix1",
        "agent": "pingou"
      }
    }


class TestProjectForked(Base):
    """ These messages are published when a someone edited a tag of a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.project.forked"
    expected_subti = 'pingou forked project "fedmsg" to "pingou/fedmsg"'
    expected_link = "https://pagure.io/fork/pingou/fedmsg"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/pingou/fedmsg'])
    msg = {
      "i": 3,
      "timestamp": 1427456769,
      "msg_id": "2015-7ec8cd76-8ed7-4360-ac32-2e881273a7c2",
      "topic": "io.pagure.dev.pagure.project.forked",
      "msg": {
        "project": {
          "description": "",
          "parent": {
            "description": "",
            "parent": None,
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "ralph",
              "emails": [
                "ralph@fedoraproject.org"
              ],
              "name": "ralph"
            },
            "date_created": "1426595173",
            "id": 5,
            "name": "fedmsg"
          },
          "project_docs": True,
          "issue_tracker": True,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          },
          "date_created": "1427453169",
          "id": 8,
          "name": "fedmsg"
        },
        "agent": "pingou"
      }
    }


class TestNewPullRequestComment(Base):
    """ These messages are published when a someone commented on a
    pull-request of a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.comment.added"
    expected_subti = 'pingou commented on pull-request#6 of '\
        'project "test"'
    expected_link = "https://pagure.io/test/pull-request/6#comment-16"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/test', 'pull-request/6'])
    msg = {
      "i": 2,
      "timestamp": 1427457362,
      "msg_id": "2015-cbf24329-b51c-4160-983c-ffa45ef63863",
      "topic": "io.pagure.dev.pagure.pull-request.comment.added",
      "msg": {
        "pullrequest": {
          "status": True,
          "branch_from": "master",
          "uid": "0a7d6b626b934511b6355dd48926916a",
          "title": "test request",
          "commit_start": "788efeaaf86bde8618f594a8181abb402e1dd904",
          "project": {
            "description": "test project",
            "parent": None,
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426500194",
            "id": 1,
            "name": "test"
          },
          "commit_stop": "5ca3e1c7ccff3327ebeb2f07eaa9bf3820d3f5c8",
          "repo_from": {
            "description": "test project",
            "parent": {
              "description": "test project",
              "parent": None,
              "project_docs": True,
              "issue_tracker": True,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "date_created": "1426500194",
              "id": 1,
              "name": "test"
            },
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426843440",
            "id": 6,
            "name": "test"
          },
          "comments": [
            {
              "comment": "This is looking good!",
              "parent": None,
              "filename": None,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "commit": None,
              "date_created": "1427453701",
              "line": None,
              "id": 16
            }
          ],
          "branch": "master",
          "date_created": "1426843718",
          "id": 6,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }

class TestEditPullRequestComment(Base):
    """ These messages are published when a someone commented on a
    pull-request of a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.comment.edited"
    expected_subti = 'lmacken edited a comment on pull-request#31 of '\
        'project "mdapi"'
    expected_link = "https://pagure.io/mdapi/pull-request/31#comment-2922"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "203f6cb95b44b5d38aa21425b066dd522d3e19d8919cf4b339f29e0ea7f03e9b" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['lmacken'])
    expected_objects = set(['project/mdapi', 'pull-request/31'])
    msg = {
      "source_name": "datanommer",
      "i": 1,
      "timestamp": 1456854211.0,
      "msg_id": "2016-79bbee62-ed68-4989-a930-d40f09edd1da",
      "topic": "io.pagure.prod.pagure.pull-request.comment.edited",
      "source_version": "0.6.5",
      "msg": {
        "comment": {
          "comment": "From https://docs.python.org/3/library/asyncio-task.html\r\n"
              "\r\nThings a coroutine can do:\r\n\r\n"
              "`result = await future` or `result = yield from future` \u2013...",
          "parent": None,
          "notification": False,
          "tree": None,
          "filename": None,
          "edited_on": "1456854211",
          "editor": {
            "fullname": "Luke Macken",
            "name": "lmacken"
          },
          "date_created": "1456854157",
          "commit": None,
          "line": None,
          "id": 2922,
          "user": {
            "fullname": "Luke Macken",
            "name": "lmacken"
          }
        },
        "project": {
          "description": "Expose the repo metadata information in a simple api",
          "parent": None,
          "settings": {
            "Minimum_score_to_merge_pull-request": -1,
            "Web-hooks": None,
            "project_documentation": False,
            "always_merge": False,
            "pull_requests": True,
            "Enforce_signed-off_commits_in_pull-request": False,
            "Only_assignee_can_merge_pull-request": False,
            "issue_tracker": True
          },
          "tags": [
            "fedmsg",
            "fedora-infra"
          ],
          "user": {
            "fullname": "Pierre-YvesChibon",
            "name": "pingou"
          },
          "date_created": "1445511838",
          "id": 221,
          "name": "mdapi"
        },
        "pullrequest": {
          "status": "Open",
          "branch_from": "more_asyncio",
          "uid": "d358fb141cab47dda84fe1cd1a19f042",
          "commit_stop": "e307cc14de0f0ab38a5fbf235e62d5d8e106dd9c",
          "title": "Chain the method as coroutines to make the process more asynchronous",
          "comments": [],
          "id": 31,
          "project": {
            "description": "Expose the repo metadata information in a simple api",
            "parent": None,
            "settings": {
              "Minimum_score_to_merge_pull-request": -1,
              "Web-hooks": None,
              "project_documentation": False,
              "always_merge": False,
              "pull_requests": True,
              "Enforce_signed-off_commits_in_pull-request": False,
              "Only_assignee_can_merge_pull-request": False,
              "issue_tracker": True
            },
            "tags": [
              "fedmsg",
              "fedora-infra"
            ],
            "user": {
              "fullname": "Pierre-YvesChibon",
              "name": "pingou"
            },
            "date_created": "1445511838",
            "id": 221,
            "name": "mdapi"
          },
          "assignee": None,
          "repo_from": {
            "description": "Expose the repo metadata information in a simple api",
            "parent": None,
            "settings": {
              "Minimum_score_to_merge_pull-request": -1,
              "Web-hooks": None,
              "project_documentation": False,
              "always_merge": False,
              "pull_requests": True,
              "Enforce_signed-off_commits_in_pull-request": False,
              "Only_assignee_can_merge_pull-request": False,
              "issue_tracker": True
            },
            "tags": [
              "fedmsg",
              "fedora-infra"
            ],
            "user": {
              "fullname": "Pierre-YvesChibon",
              "name": "pingou"
            },
            "date_created": "1445511838",
            "id": 221,
            "name": "mdapi"
          },
          "updated_on": "1456853926",
          "commit_start": "e307cc14de0f0ab38a5fbf235e62d5d8e106dd9c",
          "branch": "master",
          "date_created": "1456770725",
          "closed_at": None,
          "remote_git": None,
          "closed_by": None,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "name": "pingou"
          }
        },
        "agent": "lmacken"
      }
    }

class TestNewPullRequestclosed(Base):
    """ These messages are published when a someone closed a pull-request
    of a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.closed"
    expected_subti = 'pingou closed (without merging) pull-request#6 '\
        'of project "test"'
    expected_link = "https://pagure.io/test/pull-request/6"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/test', 'pull-request/6'])
    msg = {
      "i": 2,
      "timestamp": 1427458544,
      "msg_id": "2015-c9636fda-3a4c-452b-85ee-870e29f63a03",
      "topic": "io.pagure.dev.pagure.pull-request.closed",
      "msg": {
        "pullrequest": {
          "status": False,
          "branch_from": "master",
          "uid": "0a7d6b626b934511b6355dd48926916a",
          "title": "test request",
          "commit_start": "788efeaaf86bde8618f594a8181abb402e1dd904",
          "project": {
            "description": "test project",
            "parent": None,
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426500194",
            "id": 1,
            "name": "test"
          },
          "commit_stop": "5ca3e1c7ccff3327ebeb2f07eaa9bf3820d3f5c8",
          "repo_from": {
            "description": "test project",
            "parent": {
              "description": "test project",
              "parent": None,
              "project_docs": True,
              "issue_tracker": True,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "date_created": "1426500194",
              "id": 1,
              "name": "test"
            },
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426843440",
            "id": 6,
            "name": "test"
          },
          "comments": [
            {
              "comment": "Sorry but this won't do",
              "parent": None,
              "filename": None,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "commit": None,
              "date_created": "1427453701",
              "line": None,
              "id": 16
            },
          ],
          "branch": "master",
          "date_created": "1426843718",
          "id": 6,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou",
        "merged": False
      }
    }


class TestNewPullRequestMerged(Base):
    """ These messages are published when a someone merged a pull-request
    of a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.closed"
    expected_subti = 'pingou merged pull-request#7 of project "test"'
    expected_link = "https://pagure.io/test/pull-request/7"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/test', 'pull-request/7'])
    msg = {
      "i": 3,
      "timestamp": 1427458778,
      "msg_id": "2015-22ec6669-91fe-4c32-b324-db80fba696dd",
      "topic": "io.pagure.dev.pagure.pull-request.closed",
      "msg": {
        "pullrequest": {
          "status": False,
          "branch_from": "master",
          "uid": "d4182a2ac2d541d884742d3037c26e56",
          "title": "test request",
          "commit_start": "788efeaaf86bde8618f594a8181abb402e1dd904",
          "project": {
            "description": "test project",
            "parent": None,
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426500194",
            "id": 1,
            "name": "test"
          },
          "commit_stop": "5ca3e1c7ccff3327ebeb2f07eaa9bf3820d3f5c8",
          "repo_from": {
            "description": "test project",
            "parent": {
              "description": "test project",
              "parent": None,
              "project_docs": True,
              "issue_tracker": True,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "date_created": "1426500194",
              "id": 1,
              "name": "test"
            },
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426843440",
            "id": 6,
            "name": "test"
          },
          "comments": [
            {
              "comment": "awesome!",
              "parent": None,
              "filename": "test",
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "commit": "fa72f315373ec5f98f2b08c8ffae3645c97aaad2",
              "date_created": "1426843778",
              "line": 5,
              "id": 1
            }
          ],
          "branch": "master",
          "date_created": "1426843732",
          "id": 7,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou",
        "merged": True
      }
    }


class TestNewPullRequestNew(Base):
    """ These messages are published when a someone opens a new pull-request
    on a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.new"
    expected_subti = 'pingou opened pull-request#21: "Improve loading '\
        'speed" on project "test"'
    expected_link = "https://pagure.io/test/pull-request/21"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/test', 'pull-request/21'])
    msg = {
      "i": 4,
      "timestamp": 1427459070,
      "msg_id": "2015-1f03dc6a-3a0b-4b09-a06d-e4ca7d374729",
      "topic": "io.pagure.dev.pagure.pull-request.new",
      "msg": {
        "pullrequest": {
          "status": True,
          "branch_from": "test",
          "uid": "2bf721f0fbd34977aab78b5e1959e504",
          "title": "Improve loading speed",
          "commit_start": None,
          "project": {
            "description": "test project",
            "parent": None,
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426500194",
            "id": 1,
            "name": "test"
          },
          "commit_stop": None,
          "repo_from": {
            "description": "test project",
            "parent": {
              "description": "test project",
              "parent": None,
              "project_docs": True,
              "issue_tracker": True,
              "user": {
                "fullname": "Pierre-YvesChibon",
                "emails": [
                  "pingou@fedoraproject.org"
                ],
                "name": "pingou"
              },
              "date_created": "1426500194",
              "id": 1,
              "name": "test"
            },
            "project_docs": True,
            "issue_tracker": True,
            "user": {
              "fullname": "Pierre-YvesChibon",
              "emails": [
                "pingou@fedoraproject.org"
              ],
              "name": "pingou"
            },
            "date_created": "1426843440",
            "id": 6,
            "name": "test"
          },
          "comments": [],
          "branch": "master",
          "date_created": "1427455470",
          "id": 21,
          "user": {
            "fullname": "Pierre-YvesChibon",
            "emails": [
              "pingou@fedoraproject.org"
            ],
            "name": "pingou"
          }
        },
        "agent": "pingou"
      }
    }


class TestPullRequestFlagAdded(Base):
    """ These messages are published when a someone flags a pull-request
    on a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.flag.added"
    expected_subti = 'Jenkins flagged test#1 with "Tests failed"'
    expected_link = "https://pagure.io/test/pull-request/1"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/test', 'pull-request/1'])
    msg = {
          "username": "pingou",
          "i": 3,
          "timestamp": 1433167960,
          "msg_id": "2015-e7094e2a-1259-49da-91f5-635e81011ffa",
          "topic": "io.pagure.dev.pagure.pull-request.flag.added",
          "msg": {
            "flag": {
              "comment": "Tests failed",
              "username": "Jenkins",
              "uid": "jenkins_build_pagure_100+seed",
              "url": "http://jenkins.cloud.fedoraproject.org/",
              "percent": "0",
              "pull_request_uid": "cb0cc178203046fe86f675779b31b913",
              "user": {
                "fullname": "PY C",
                "default_email": "bar@pingou.com",
                "emails": [
                  "bar@pingou.com",
                  "foo@pingou.com"
                ],
                "name": "pingou"
              },
              "date_created": "1433160759"
            },
            "pullrequest": {
              "status": True,
              "branch_from": "master",
              "uid": "cb0cc178203046fe86f675779b31b913",
              "title": "test pull-request",
              "commit_start": None,
              "project": {
                "description": "test project #1",
                "parent": None,
                "settings": {
                  "Minimum_score_to_merge_pull-request": -1,
                  "Web-hooks": None,
                  "project_documentation": True,
                  "pull_requests": True,
                  "Only_assignee_can_merge_pull-request": False,
                  "issue_tracker": True
                },
                "user": {
                  "fullname": "PY C",
                  "default_email": "bar@pingou.com",
                  "emails": [
                    "bar@pingou.com",
                    "foo@pingou.com"
                  ],
                  "name": "pingou"
                },
                "date_created": "1433160759",
                "id": 1,
                "name": "test"
              },
              "commit_stop": None,
              "repo_from": {
                "description": "test project #1",
                "parent": None,
                "settings": {
                  "Minimum_score_to_merge_pull-request": -1,
                  "Web-hooks": None,
                  "project_documentation": True,
                  "pull_requests": True,
                  "Only_assignee_can_merge_pull-request": False,
                  "issue_tracker": True
                },
                "user": {
                  "fullname": "PY C",
                  "default_email": "bar@pingou.com",
                  "emails": [
                    "bar@pingou.com",
                    "foo@pingou.com"
                  ],
                  "name": "pingou"
                },
                "date_created": "1433160759",
                "id": 1,
                "name": "test"
              },
              "assignee": None,
              "comments": [],
              "branch": "master",
              "date_created": "1433160759",
              "id": 1,
              "user": {
                "fullname": "PY C",
                "default_email": "bar@pingou.com",
                "emails": [
                  "bar@pingou.com",
                  "foo@pingou.com"
                ],
                "name": "pingou"
              }
            },
            "agent": "pingou"
          }
        }


class TestPullRequestFlagUpdated(Base):
    """ These messages are published when a someone updates a flag on a
    pull-request on a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.pull-request.flag.updated"
    expected_subti = 'Jenkins updated the flags on test#1 with: '\
        '"Tests passed"'
    expected_link = "https://pagure.io/test/pull-request/1"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/test', 'pull-request/1'])
    msg = {
          "username": "pingou",
          "i": 4,
          "timestamp": 1433167960,
          "msg_id": "2015-e7094e2a-1259-49da-91f5-635e81011ffa",
          "topic": "io.pagure.dev.pagure.pull-request.flag.updated",
          "msg": {
            "flag": {
              "comment": "Tests passed",
              "username": "Jenkins",
              "uid": "jenkins_build_pagure_101+seed",
              "url": "http://jenkins.cloud.fedoraproject.org/",
              "percent": "100",
              "pull_request_uid": "cb0cc178203046fe86f675779b31b913",
              "user": {
                "fullname": "PY C",
                "default_email": "bar@pingou.com",
                "emails": [
                  "bar@pingou.com",
                  "foo@pingou.com"
                ],
                "name": "pingou"
              },
              "date_created": "1433160759"
            },
            "pullrequest": {
              "status": True,
              "branch_from": "master",
              "uid": "cb0cc178203046fe86f675779b31b913",
              "title": "test pull-request",
              "commit_start": None,
              "project": {
                "description": "test project #1",
                "parent": None,
                "settings": {
                  "Minimum_score_to_merge_pull-request": -1,
                  "Web-hooks": None,
                  "project_documentation": True,
                  "pull_requests": True,
                  "Only_assignee_can_merge_pull-request": False,
                  "issue_tracker": True
                },
                "user": {
                  "fullname": "PY C",
                  "default_email": "bar@pingou.com",
                  "emails": [
                    "bar@pingou.com",
                    "foo@pingou.com"
                  ],
                  "name": "pingou"
                },
                "date_created": "1433160759",
                "id": 1,
                "name": "test"
              },
              "commit_stop": None,
              "repo_from": {
                "description": "test project #1",
                "parent": None,
                "settings": {
                  "Minimum_score_to_merge_pull-request": -1,
                  "Web-hooks": None,
                  "project_documentation": True,
                  "pull_requests": True,
                  "Only_assignee_can_merge_pull-request": False,
                  "issue_tracker": True
                },
                "user": {
                  "fullname": "PY C",
                  "default_email": "bar@pingou.com",
                  "emails": [
                    "bar@pingou.com",
                    "foo@pingou.com"
                  ],
                  "name": "pingou"
                },
                "date_created": "1433160759",
                "id": 1,
                "name": "test"
              },
              "assignee": None,
              "comments": [],
              "branch": "master",
              "date_created": "1433160759",
              "id": 1,
              "user": {
                "fullname": "PY C",
                "default_email": "bar@pingou.com",
                "emails": [
                  "bar@pingou.com",
                  "foo@pingou.com"
                ],
                "name": "pingou"
              }
            },
            "agent": "pingou"
          }
        }


class LegacyTestGitCommit(Base):
    """ These messages are published when a someone updates a flag on a
    pull-request on a project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.git.receive"
    expected_subti = 'pingou pushed to pagure (bleach). '\
        '"Use the default attributes for bleach and then add our own"'
    expected_link = "https://pagure.io/pagure/131e07ed2538839d509880a011cd7d55c0967171"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = None
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/pagure'])
    msg = {
        'i': 1,
        'msg': {
            'commit': {
                'agent': 'git',
                'branch': 'refs/heads/bleach',
                'email': 'pingou@fedoraproject.org',
                'message': 'Use the default attributes for bleach and then add our own',
                'name': 'Pierre-Yves Chibon',
                'path': '/srv/git/repositories/pagure.git',
                'repo': {
                    'date_created': '1431549490',
                    'description': 'A git centered forge',
                    'id': 10,
                    'name': 'pagure',
                    'parent': None,
                    'settings': {
                        'Minimum_score_to_merge_pull-request': -1,
                        'Only_assignee_can_merge_pull-request': False,
                        'Web-hooks': None,
                        'issue_tracker': True,
                        'project_documentation': True,
                        'pull_requests': True},
                        'user': {
                            'fullname': 'Pierre-YvesChibon',
                            'name': 'pingou'
                        }
                },
                'rev': '131e07ed2538839d509880a011cd7d55c0967171',
                'seen': False,
                'stats': {
                    'files': {
                        'pagure/ui/filters.py': {
                            'additions': 4,
                            'deletions': 4,
                            'lines': 8
                        }
                    },
                    'total': {
                        'additions': 4,
                        'deletions': 4,
                        'files': 1,
                        'lines': 8
                    }
                },
                'summary': 'Use the default attributes for bleach and then add our own',
                'username': None}},
         'msg_id': '2015-c6561fe2-3815-469c-a288-8f36cf8fdda3',
         'source_name': 'datanommer',
         'source_version': '0.6.5',
         'timestamp': 1434213042.0,
         'topic': 'io.pagure.prod.pagure.git.receive'
    }


class TestGitCommit(Base):
    """ These messages are published when a someone pushes a commit to a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.git.receive"
    expected_subti = 'pingou pushed 3 to pagure (master)'
    expected_link = "https://pagure.io/pagure/branch/master"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = 'https://seccdn.libravatar.org/avatar/' + \
        '01fe73d687f4db328da1183f2a1b5b22962ca9d9c50f0728aafeac974856311c' + \
        '?s=64&d=retro'
    expected_packages = set([])
    expected_usernames = set(['pingou'])
    expected_objects = set(['project/pagure'])
    msg = {
      "username": "pingou",
      "i": 1,
      "timestamp": 1457538778,
      "msg_id": "2016-c854f690-5691-42e8-b488-2d65aef80fdc",
      "topic": "io.pagure.prod.pagure.git.receive",
      "msg": {
        "forced": False,
        "agent": "pingou",
        "repo": {
          "description": "test project #1",
          "parent": None,
          "settings": {
            "Minimum_score_to_merge_pull-request": -1,
            "Web-hooks": None,
            "project_documentation": False,
            "always_merge": True,
            "pull_requests": True,
            "Enforce_signed-off_commits_in_pull-request": False,
            "Comment-editing": False,
            "Only_assignee_can_merge_pull-request": False,
            "issue_tracker": True
          },
          "tags": [
            "fedora-infra",
            "fedora"
          ],
          "user": {
            "fullname": "Pierre-YvesChibon",
            "name": "pingou"
          },
          "date_created": "1426500194",
          "id": 1,
          "name": "pagure"
        },
        "end_commit": "edc02fbb423d3957d174c571896418f29fa169b8",
        "branch": "refs/heads/master",
        "total_commits": 3,
        "start_commit": "b5e65479e4bd91554d8d3084bf378ffb6e4ef605"
      }
    }





class TestIssueDrop(Base):
    """ These messages are published when a ticket is deleted against a
    project on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.drop"
    expected_subti = 'yangl1996 deleted ticket docs-test#42: "foobar"'
    expected_link = "https://pagure.io/docs-test/issue/42"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "4535d62cac804e6e0cb25386c9a7701e680deb176efa79829d076fc63c2b96e7" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['yangl1996'])
    expected_objects = set(['project/docs-test', 'issue/42'])
    msg = {
        'source_name': 'datanommer',
        'i': 1,
        'timestamp': 1434790435.0,
        'msg_id': '2015-9ab3ba70-02aa-4034-a532-8791696165fd',
        'topic': 'io.pagure.prod.pagure.issue.drop',
        'source_version': '0.6.5',
        'msg': {
            'project': {
                'description': '',
                'parent': None,
                'settings': {
                    'Minimum_score_to_merge_pull-request': '-1',
                    'Web-hooks': 'http://128.199.82.190:7655',
                    'project_documentation': 'y',
                    'pull_requests': 'y',
                    'Enforce_signed-off_commits_in_pull-request': False,
                    'Only_assignee_can_merge_pull-request': False,
                    'issue_tracker': 'y'
                },
                'user': {
                    'fullname': 'Lei Yang',
                    'name': 'yangl1996'
                },
                'date_created': '1434262409',
                'id': 78,
                'name': 'docs-test'
            },
            'issue': {
                'status': 'Open',
                'blocks': [],
                'tags': [],
                'title': 'foobar',
                'comments': [],
                'content': 'report',
                'assignee': None,
                'depends': [],
                'private': False,
                'date_created': '1434789890',
                'id': 42,
                'user': {
                    'fullname': 'Lei Yang',
                    'name': 'yangl1996'
                }
            },
            'agent': 'yangl1996'
        }
    }


class TestIssueCommentEdit(Base):
    """ These messages are published when someone edits a comment on a ticket
    on `pagure <https://pagure.io>`_.
    """
    expected_title = "pagure.issue.comment.edited"
    expected_subti = 'adamwill edited a comment on ticket (unknown)#5: ' + \
        '"Don\'t use `-` characters in metadata values"'
    expected_link = "https://pagure.io"
    expected_icon = "https://apps.fedoraproject.org/packages/" + \
        "images/icons/package_128x128.png"
    expected_secondary_icon = "https://seccdn.libravatar.org/avatar/" + \
        "0e6c01c694d74afb0cbb65bc1be91950111503818caf492c0938dc2b97c48c41" + \
        "?s=64&d=retro"
    expected_packages = set([])
    expected_usernames = set(['adamwill'])
    expected_objects = set(['project/(unknown)', 'issue/5'])
    msg = {
        "i": 2,
        "msg": {
            "agent": "adamwill",
            "comment": {
                "comment": "For reference, the practical case I have here is "
                "that for openQA purposes I need to create a `flavor`, "
                "which is something like \"the server boot iso\" or \"the "
                "cloud atomic qcow disk image\". The logical way to do this "
                "is to combine a few Pungi metadata values; right now I'm "
                "using `variant` plus `type` plus `format` (format seems to "
                "be necessary to distinguish, say, 'cloud base qcow2' from "
                "'cloud base img' or whatever).\r\n\r\nIf `variant` can be "
                "`Cloud_Atomic` and `format` can be `raw-xz`, what character "
                "am I supposed to use to join the values such that I can "
                "reliably split them up again later (which I also need to "
                "do)?",
                "date_created": "1454009026",
                "edited_on": "1454009050",
                "editor": {
                    "fullname": "Adam Williamson",
                    "name": "adamwill",
                },
                "id":1489,
                "parent":None,
                "user": {
                    "fullname": "Adam Williamson",
                    "name": "adamwill",
                },
            },
            "issue": {
                "assignee":None,
                "blocks":[],
                "comments":[],
                "content": "So I'm looking at "
                "https://pagure.io/pungi-fedora/blob/master/f/fedora."
                "conf#_270 :\r\n\r\n                'format': [('"
                "qcow2','qcow2'), ('raw-xz','raw.xz')]\r\n\r\nI'm "
                "guessing the value `'raw-xz'` is what would show up in the "
                "metadata as the `format` of that image.\r\n\r\nHowever, in "
                "other metadata fields - e.g. the arch, `x86_64`, and I "
                "think some variants, e.g. `Cloud_Atomic` - we use _ to "
                "separate words inside values.\r\n\r\nHaving some fields "
                "use `_` as an internal separator and others use `-` as an "
                "internal separator is going to be *very* messy; people tend "
                "to join values with one character or the other, but if both "
                "characters are used within values, splitting them back out "
                "again becomes impossible or messy.\r\n\r\nThe [naming "
                "policy](https://fedoraproject.org/wiki/User:Adamwill/Draft_"
                "fedora_image_naming_policy) explicitly covers this and "
                "forbids the use of `-` within values:\r\n\r\n\"Fields may "
                "only contain ASCII alphanumeric characters and underscores. "
                "Particularly of note, the character '-' is reserved for use "
                "as a field separator. No field may itself contain that "
                "character.\"\r\n\r\nSo, can we please change `raw-xz` to "
                "`raw_xz` and generally make it a rule that the metadata "
                "values don't use `-`? Thanks!\r\n\r\n(I see also `distro` "
                "and `name` fields that use `-`, but I'm not sure either of "
                "those actually winds up in the output metadata, which is "
                "what I care about).",
                "date_created": "1454008733",
                "depends":[],
                "id":5,
                "private":False,
                "status": "Open",
                "tags":[],
                "title": "Don't use `-` characters in metadata values",
                "user": {
                    "fullname": "Adam Williamson",
                    "name": "adamwill",
                },
            },
        },
        "msg_id": "2016-6e2bb7ae-9af1-460d-ad7d-ff5bf12df989",
        "source_name": "datanommer",
        "source_version": "0.6.5",
        "timestamp":1454009050.0,
        "topic": "io.pagure.prod.pagure.issue.comment.edited",
    }



add_doc(locals())

if __name__ == '__main__':
    unittest.main()



import fedmsg.meta.base
from fedmsg_meta_fedora_infrastructure.fasshim import avatar_url_from_email


class ByMessageId(fedmsg.meta.base.BaseConglomerator):
    def can_handle(self, msg, **config):
        return '.mailman.receive' in msg['topic']

    def matches(self, a, b, **config):
        a, b = a['msg']['msg'], b['msg']['msg']
        a_references, b_references = [], []
        if a['references']:
            a_references = [r.strip() for r in a['references'].split('\n')]
        if b['references']:
            b_references = [r.strip() for r in b['references'].split('\n')]

        a_references = set(a_references + [a['message-id']])
        b_references = set(b_references + [b['message-id']])

        # Return True if the messages share some subset of references.
        return a_references & b_references

    def get_secondary_icon(self, constituents, default):
        agents = set([m['msg']['msg']['from'] for m in constituents])

        # Just take the first.
        agent = sorted(agents)[0]

        email = agent.split('<')[-1].split('>')[0]

        return avatar_url_from_email(email)

    def get_link(self, constituents):
        links = set([m['msg']['msg']['archived-at'] for m in constituents])

        # Just take the first.
        link = sorted(links)[0]

        link = link.split('<')[-1].split('>')[0]

        return link

    def merge(self, constituents, subject, **config):
        ms = constituents  # shorthand

        subtitle = u'{people} wrote {N} replies to the ' + \
            '"{subject}" thread on the {mlists} {predicate}'

        people = set([
            m['msg']['msg']['from'].split('<')[-1].split('>')[0]
            for m in ms
        ])
        mlists = set([m['msg']['mlist']['list_name'] for m in ms])
        predicate = 'list' if len(mlists) == 1 else 'lists'
        people = self.list_to_series(people)
        mlists = self.list_to_series(mlists)

        # Build a set of subjects and strip off the leading 'Re: RE: re: ...'
        subjects = set()
        for m in ms:
            subject = m['msg']['msg']['subject']
            if 're: ' in subject.lower():
                index = subject.lower().rindex('re: ') + 4
                subject = subject[index:]
            subjects.add(subject)

        # Just take the first....
        subject = sorted(subjects)[0]

        tmpl = self.produce_template(constituents, subject, **config)
        tmpl['subtitle'] = subtitle.format(
            people=people,
            N=len(ms),
            subject=subject,
            mlists=mlists,
            predicate=predicate,
        )
        tmpl['subjective'] = tmpl['subtitle']

        default = tmpl['icon']

        tmpl['secondary_icon'] = self.get_secondary_icon(constituents, default)
        tmpl['link'] = self.get_link(constituents)

        return tmpl



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Marek Brysa <mbrysa@redhat.com>

from fedmsg_meta_fedora_infrastructure import BaseProcessor

_long_template = """Packages: {components}
Function: {function}
First occurrence: {first_occurrence}
Type:     {type}
Count:    {count}
URL:      {url}
"""


class FAFProcessor(BaseProcessor):
    __name__ = "faf"
    __description__ = "ABRT crash analysis server"
    __link__ = "https://retrace.fedoraproject.org"
    __obj__ = "New and significant crashes"
    __docs__ = "https://github.com/abrt/faf/wiki"
    __icon__ = "https://apps.fedoraproject.org/packages/images/icons/abrt.png"

    def long_form(self, msg, **config):
        return _long_template.format(**{
            "components": ", ".join(msg["msg"].get("components", [])),
            "first_occurrence": msg["msg"].get("first_occurrence"),
            "function": msg["msg"].get("function"),
            "type": msg["msg"].get("type"),
            "count": msg["msg"].get("count"),
            "url": msg["msg"].get("url"),
            })

    def subtitle(self, msg, **config):
        obj_name = "report"
        if "problem.threshold" in msg["topic"]:
            obj_name = "problem"
        return ("ABRT {0} for package {1} has reached {2} occurrences"
                .format(obj_name, ", ".join(msg["msg"].get("components", [])),
                        msg["msg"].get("count")))

    def link(self, msg, **config):
        return msg["msg"].get("url")

    def usernames(self, msg, **config):
        return set(msg["msg"].get("maintainers", []))

    def packages(self, msg, **config):
        return set(msg["msg"].get("components", []))

    def secondary_icon(self, msg, **config):
        pkgs = msg["msg"].get("components", [])
        if not pkgs:
            return None
        return ("https://apps.fedoraproject.org/packages/images/icons/{0}.png"
                .format(pkgs[0]))



# This file is part of fedmsg.
# Copyright (C) 2012-2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class ZanataProcessor(BaseProcessor):
    __name__ = "Zanata"
    __description__ = "translation events"
    __link__ = "https://fedora.zanata.org"
    __docs__ = "http://zanata.org/help/workflow-overview/"
    __obj__ = "Translation Events"
    __icon__ = "https://pbs.twimg.com/profile_images/" + \
        "378800000417679469/47eb45c6205aa9f2cdb8705e6d46745c_normal.png"

    def subtitle(self, msg, **config):
        tmpl = self._(
            "{docId} from the {project} project "
            "is now {milestone} in the '{locale}' locale"
        )
        return tmpl.format(**msg['msg']).lower()

    def secondary_icon(self, msg, **config):
        return self.__icon__

    def _object(self, msg):
        return "/".join([
            msg['msg']['project'],
            msg['msg']['version'],
            'languages',
            msg['msg']['locale'],
        ])

    def objects(self, msg, **config):
        return set([self._object(msg)])

    def link(self, msg, **config):
        return "https://fedora.zanata.org/iteration/view/" + self._object(msg)



# This file is part of fedmsg.
# Copyright (C) 2015 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors: Ralph Bean <rbean@redhat.com>
#
""" Tests for infra git messages (ansible, dns, etc..) """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from .common import add_doc


class TestInfraGitPuppet(Base):
    """ These messages get published by git repos owned by the Fedora
    Infrastructure team.  These are the repos we use to manage the systems that
    run fedoraproject.org.

    In particular, this message is an example of a message from our "puppet"
    repo, which is older and being retired in favor of our ansible repo.
    """
    expected_title = "infragit.receive"
    expected_subti = 'ralph pushed a commit to the fedora-infra puppet ' + \
        'repo (master):  "Testing again for fedmsg."'
    expected_link = ''  # No links for puppet
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = ("https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_usernames = set(['ralph'])
    expected_agent = 'ralph'
    expected_packages = set()
    expected_objects = set(['puppet/test'])
    msg = {
        "i": 1,
        "timestamp": 1439584387.0,
        "msg_id": "2015-ee126d0a-a89b-4a09-ba15-4ca3382eb8fe",
        "topic": "org.fedoraproject.prod.infragit.receive",
        "source_version": "0.6.5",
        "msg": {
            "commit": {
                "username": "ralph",
                "stats": {
                    "files": {
                        "test": {
                            "deletions": 0,
                            "additions": 0,
                            "lines": 0
                        }
                    },
                    "total": {
                        "deletions": 0,
                        "files": 1,
                        "additions": 0,
                        "lines": 0
                    }
                },
                "name": "Ralph Bean",
                "rev": "6e7177a1d5fd712cb53ce70acb17b92c5f791f08",
                "agent": "ralph",
                "summary": "Testing again for fedmsg.",
                "repo": "",
                "branch": "master",
                "seen": False,
                "path": "/git/puppet",
                "message": "Testing again for fedmsg.\n",
                "email": "rbean@redhat.com"
            }
        }
    }


class TestInfraGitAnsible(Base):
    """ These messages get published by git repos owned by the Fedora
    Infrastructure team.  These are the repos we use to manage the systems that
    run fedoraproject.org.

    In particular, this message is an example of a message from our "ansible"
    repo, which is where we do most of our work.
    """
    expected_title = "infragit.receive"
    expected_subti = 'ralph pushed a commit to the fedora-infra ansible ' + \
        'repo (master):  "Testing again for fedmsg."'
    expected_link = "https://infrastructure.fedoraproject.org/cgit/" + \
        "ansible.git/commit/" + \
        "?h=master&id=6e7177a1d5fd712cb53ce70acb17b92c5f791f08"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = ("https://seccdn.libravatar.org/avatar/"
        "9c9f7784935381befc302fe3c814f9136e7a33953d0318761669b8643f4df55c"
        "?s=64&d=retro")
    expected_usernames = set(['ralph'])
    expected_agent = 'ralph'
    expected_packages = set()
    expected_objects = set(['ansible/test'])
    msg = {
        "i": 1,
        "timestamp": 1439584387.0,
        "msg_id": "2015-ee126d0a-a89b-4a09-ba15-4ca3382eb8fe",
        "topic": "org.fedoraproject.prod.infragit.receive",
        "source_version": "0.6.5",
        "msg": {
            "commit": {
                "username": "ralph",
                "stats": {
                    "files": {
                        "test": {
                            "deletions": 0,
                            "additions": 0,
                            "lines": 0
                        }
                    },
                    "total": {
                        "deletions": 0,
                        "files": 1,
                        "additions": 0,
                        "lines": 0
                    }
                },
                "name": "Ralph Bean",
                "rev": "6e7177a1d5fd712cb53ce70acb17b92c5f791f08",
                "agent": "ralph",
                "summary": "Testing again for fedmsg.",
                "repo": "",
                "branch": "master",
                "seen": False,
                "path": "/git/ansible",
                "message": "Testing again for fedmsg.\n",
                "email": "rbean@redhat.com"
            }
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()
